/*! @file    main.c
 *  @brief   AWU唤醒
 *  @version 2021-06-30  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "nccm0670_exti.h"
#include "nccm0670_eval.h"
#include "nccm0670_gpio.h"
#include "nccm0670_awu.h"
#include "nccm0670_pwr.h"
#include "nccm0670_rcc.h"
#include "systick.h"

static volatile uint8_t awu_num = 0;

int main(void)
{
    RCC_EnablePeripheralClock(RCC_PWR);
    RCC_EnablePeripheralClock(RCC_AWU);
    /* 关闭其它中断，防止异常唤醒 */
    __disable_irq();

    /* 关系统嘀嗒 */
    SysTick->CTRL &= ~SysTick_CTRL_ENABLE_Msk;
    NVIC_DisableIRQ(USART1_IRQn);

    exti_config_t exti_cfg;
    exti_cfg.mode = EXTI_MODE_INTERRUPT;
    exti_cfg.trigger = EXTI_TRIGGER_RISING;
    /* 设置awu外部中断线20 */
    EXTI_SetConfigLine(EXTI_LINE_20, &exti_cfg);
    /* 开awu中断 */
    NVIC_EnableIRQ(AWU_IRQn);

    __enable_irq();

    RCC_EnableOSC(RCC_OSC_LSI);
    awu_config_t cfg = {
        .awu_timeout = 0,
        .buz_frequency = 0,
    };
    /* 唤醒时间为：16384/32000*(x+1) 秒，此处设置约为500ms */
    AWU_Init(&cfg);
    AWU_Enable();
    /* 进入低功耗模式 */
    PWR_EnterStopMode(PWR_LOWPOWERREGULATOR_ON, PWR_LOWPOWER_ENTRY_WFI);

    /* 重新打开关闭的中断 */
    SysTick->CTRL |= SysTick_CTRL_ENABLE_Msk;
    NVIC_EnableIRQ(USART1_IRQn);

    ncc_eval_com_init();

    while (1)
    {
        /* 表示进中断了 */
        if (awu_num == 1)
        {
            awu_num = 0;
            printf("wake up successfully\r\n");
        }
    }
}

void AWU_IRQHandler(void)
{
    awu_num = 1;
}
