/*! @file    main.c
 *  @brief   GTIM定时
 *  @version 2021-06-30  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "nccm0670_eval.h"
#include "nccm0670_gtim.h"
#include "nccm0670_rcc.h"

void rcc_config(void);
void gpio_config(void);
void timer_base_init(void);
void timer_configuration(void);

int main(void)
{
    ncc_eval_led_init(LED1);
    RCC_EnablePeripheralClock(RCC_TIM2);

    timer_configuration();
    while (1)
    {
    }
}

void rcc_config(void)
{
    RCC_EnablePeripheralClock(RCC_GPIOA);
    RCC_EnablePeripheralClock(RCC_TIM2);
}
void timer_base_init(void)
{
    gtim_base_config_t gtim_cfg = {0};
    /* 向上计数模式 */
    gtim_cfg.counter_mode       = GTIM_COUNTER_UP;
    /* 定时器时钟分频 */
    gtim_cfg.clock_division     = GTIM_CLOCK_DIVISION_DIV1;
    /* 计数器预分频系数设为 1000 */
    gtim_cfg.prescaler          = 999;
    /* 计数器重载值设为 4000，更新频率为4K/4000 = 1Hz */
    gtim_cfg.period             = 3999;
    GTIM_Init(TIM2, &gtim_cfg);
}

void timer_configuration(void)
{
    timer_base_init();
    /*使能外设中断，每隔一秒钟发生一次计数器上溢中断 */
    GTIM_SetIntMode(TIM2, GTIM_INT_UIE, ENABLE);
    GTIM_Enable(TIM2);
    /*设置NVIC中断*/
    NVIC_SetPriority(TIM2_IRQn, 0);
    NVIC_EnableIRQ(TIM2_IRQn);
}

void TIM2_IRQHandler(void)
{
    /* 更新中断标志位有效 && 更新中断使能 */
    if (GTIM_GetIntFlag(TIM2, GTIM_FLAG_UIF))
    {
        /* 清空更新中断标志位 */
        GTIM_ClearStatusFlag(TIM2, GTIM_FLAG_UIF);
        /* 翻转 LED1 */
        ncc_eval_led_toggle(LED1);
    }
}