/*! @file    main.c
 *  @brief   IWDG例程
 *  @version 2021-06-29  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "nccm0670_eval.h"
#include "nccm0670_iwdg.h"
#include "nccm0670_rcc.h"
#include "systick.h"

void rcc_config();
void check_iwdg_reset(void);
void iwdg_config(void);

int main(void)
{
    ncc_eval_led_init(LED1);
    ncc_eval_led_init(LED2);

    ncc_eval_led_on(LED1);
    ncc_eval_led_on(LED2);

    systick_init();
    ncc_eval_com_init();

    /*开看门狗低频时钟*/
    rcc_config();
    /* 检查是否为独立看门狗复位启动 */
    check_iwdg_reset();
    /*独立看门狗初始化*/
    iwdg_config();

    while (1)
    {
        ncc_eval_led_toggle(LED2);
        /* 插入延迟时间，修改为1300验证看门狗重启 */
        delay_ms(1200);

        /* 刷新IWDG */
        IWDG_Refresh();
        printf("fresh iwdg\r\n");
    }
}

void rcc_config(void)
{
    RCC_EnableOSC(RCC_OSC_LSI);
}
void check_iwdg_reset(void)
{
    /* 是否为独立看门狗复位启动 */
    if (RCC_GetResetReason() & RCC_FLAG_IWDGRST)
    {
        /* 清除复位标志 */
        RCC_ClearAllResetFlag();
        printf("iwdg reset\r\n");
        while (1)
        {
        }
    }
    else
    {
        delay_ms(150);
        ncc_eval_led_off(LED1);
    }
}

void iwdg_config(void)
{
    iwdg_config_t wdg_cfg = {0};
    /*设置看门狗分频和装载值,看门狗超时时间 = 64k / 4 / 20000 = 0.8hz (1.25s)*/
    wdg_cfg.prescaler    = IWDG_PRESCALER_4;
    wdg_cfg.reload_value = 20000;
    /*初始化，使能看门狗*/
    IWDG_Init(&wdg_cfg);
    IWDG_Enable();
}