/*! @file    main.c
 *  @brief   TK例程
 *  @version 2021-06-28  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "nccm0670_adc.h"
#include "nccm0670_eval.h"
#include "nccm0670_rcc.h"
#include "nccm0670_tk.h"
#include "systick.h"

void tk_test_init(uint16_t channel);
void tk_display(uint16_t threshold);

/* 触摸通道选择 */
#define TK_CHANNEL TK_CHANNEL_1 | TK_CHANNEL_4 | TK_CHANNEL_5
uint16_t tk_data[16];

int main(void)
{  
    /* SYSTICK初始化 */
    systick_init();
    /* 调试串口初始化 */
    ncc_eval_com_init();
    ncc_eval_led_init(LED1);

    tk_test_init(TK_CHANNEL);
    TK_Enable();

    while (1)
    {
        /*等待触摸转换完成*/
        while (!TK_GetStatusFlag(TK_FLAG_CHARGE_FINISHED))
        {
        }
        TK_ClearStatusFlag(TK_FLAG_CHARGE_FINISHED);

        /*读取当前ADC数据对应的TK通道序号*/
        uint16_t tk_ch = TK_GetWorkingChannel();
        /*读取ADC转换结果值*/
        tk_data[tk_ch] = ADC_GetValue();
        /*触摸显示,触摸阈值可以根据实际情况设置，触摸前后的ADC读取值会相差几百*/
        tk_display(2300);
    }
}

void tk_test_init(uint16_t channel)
{
    adc_config_t adc_cfg    = { 0 };
    RCC_EnablePeripheralClock(RCC_GPIOB);
    RCC_EnablePeripheralClock(RCC_TK);
    RCC_EnablePeripheralClock(RCC_ADC);
    
    /*选择触控通道*/
    TK_SetChannels(channel);    
    /*硬件自动触发ADC转换*/                 
    TK_SetTrigMode(TK_TRIG_HARDWARE); 
    /*选择内部高速时钟48MHz*/            
    TK_SetClockSource(TK_CLK_SRC_HSI); 
    /*充电次数 64 * (TK_CRGNUM[7:0] + 1)次*/           
    TK_SetChargeCycles(99);  
    /*间隔时间1ms*/
    TK_SetChargeInterval(TK_CHARGE_INTERVAL_1MS); 
    /*工作时钟6分频*/
    TK_SetChargeFreq(TK_CHARGE_FREQ_DIV_6); 
    /*硬件自动触发ADC转换模式时提前使能ADC时间*/      
    TK_SetHwTrigLeadTime(0xff);                   
    
    /*默认：单次转换*/
    adc_cfg.continuous_conv = ADC_CONV_SINGLE; 
    /*默认：2分频*/     
    adc_cfg.prescaler       = ADC_CLOCK_DIV2;      
    /*AVDD 5V*/ 
    adc_cfg.vref            = ADC_VREF_AVDD;       
    /*默认：上升沿触发*/ 
    adc_cfg.edge            = ADC_TRIG_EDGE_RISING; 
    /*默认：内部软件触发*/
    adc_cfg.trigger         = ADC_TRIG_SOFT;        
    ADC_Init(&adc_cfg);

}

void tk_display(uint16_t threshold)
{
    for (int i = 0; i < 16; i++)
    {
        /*判断触摸值是否大于设定值*/
        if(tk_data[i] > threshold)
        {
            ncc_eval_led_on(LED1);
        }
        else
        {
            ncc_eval_led_off(LED1);
        }
        printf("ch%d : %d\r\n", i, tk_data[i]);
    }
    printf("\r\n");
}
