/*! @file    main.c
 *  @brief   WWDG例程
 *  @version 2021-06-28  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "nccm0670_eval.h"
#include "nccm0670_rcc.h"
#include "nccm0670_wwdg.h"
#include "systick.h"

void rcc_config(void);
void check_wwdg_reset(void);
void wwdg_config(void);

int main(void)
{
    ncc_eval_led_init(LED1);
    ncc_eval_led_init(LED2);

    ncc_eval_led_on(LED1);
    ncc_eval_led_on(LED2);

    systick_init();
    ncc_eval_com_init();

    /* 打开窗口看门狗时钟 */
    rcc_config();
    /* 检查是否为窗口看门狗复位启动 */
    check_wwdg_reset();
    /* 窗口看门狗初始化 */
    wwdg_config();

    while (1)
    {
        ncc_eval_led_toggle(LED2);
        /* 插入延迟时间，修改为300验证看门狗重启 */
        delay_ms(400);

        /* 刷新WWDG */
        WWDG_SetCounter(127);
        printf("fresh wwdg\n");
    }
}

void rcc_config(void)
{
    RCC_EnablePeripheralClock(RCC_WWDG);
}
void check_wwdg_reset(void)
{
    /* 是否为窗口看门狗复位启动 */
    if (RCC_GetResetReason() & RCC_FLAG_WWDGRST)
    {
        /* 清除复位标志 */
        RCC_ClearAllResetFlag();
        printf("wwdg reset\n");
    }
    else
    {
        delay_ms(150);
        ncc_eval_led_off(LED1);
    }
}
void wwdg_config(void)
{
    wwdg_config_t wdg_cfg = {0};
    /* WWDG计数时钟 = PCLK (4MHz)/4096)/8 = 122Hz (~8.192 ms)
       设置计数值为127，WWDG超时时间 = 8.192 * (127-63) = 524.288 ms
       窗口值80，这样允许刷新的时间范围是：8.192 * (127-80)= 385.024ms < 刷新窗口 < 8.192 * 64 =524.288ms */
    wdg_cfg.prescaler = WWDG_PRESCALER_8;
    wdg_cfg.window    = 80;
    wdg_cfg.counter   = 127;
    /*初始化，使能WWDG*/
    WWDG_Init(&wdg_cfg);
    WWDG_Enable();
}
