/*! @file    nccm0670_adc.h
 *  @brief   ADC固件库
 *  @version 2022-11-03  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_ADC_H
#define NCC0670_ADC_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_adc
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/*!
 * @brief   参考电压选择
 */
typedef enum
{
    ADC_VREF_AVDD              = ADC_CTL_VREF_SEL(0),                       /*!< AVDD 5V  */
    ADC_VREF_EXTERNAL          = ADC_CTL_VREF_SEL(1),                       /*!< 外部参考电压 */
    ADC_VREF_INTERNAL_VBUF_1V8 = ADC_CTL_VREF_SEL(2) | ADC_CTL_VBUF_SEL(1), /*!< 内部BUF电压1.8V*/
    ADC_VREF_INTERNAL_VBUF_2V5 = ADC_CTL_VREF_SEL(2) | ADC_CTL_VBUF_SEL(2), /*!< 内部BUF电压2.5V*/
    ADC_VREF_INTERNAL_VBUF_3V  = ADC_CTL_VREF_SEL(2) | ADC_CTL_VBUF_SEL(3)  /*!< 内部BUF电压3V*/
} adc_voltage_reference_t;

/*!
 * @brief   ADC参考阈值比较模式
 */
typedef enum
{
    ADC_COMPARE_MODE_OFF,        /*!< 禁止阈值比较 */
    ADC_COMPARE_MODE_LOW_LIMIT,  /*!< 下限，ADC数据低于低阈值数据 */
    ADC_COMPARE_MODE_BOTH_LIMIT, /*!< 区间，ADC数据在低阈值及高阈值数据之间 */
    ADC_COMPARE_MODE_HIGH_LIMIT  /*!< 上限，ADC数据高于高阈值数据 */
} adc_compare_mode_t;

/*!
 * @brief   采样时间
 */
typedef enum
{
    ADC_SAMPLETIME_1CYCLE,   /*!< 采样周期为1个周期 */
    ADC_SAMPLETIME_2CYCLES,  /*!< 采样周期为2个周期 */
    ADC_SAMPLETIME_3CYCLES,  /*!< 采样周期为3个周期 */
    ADC_SAMPLETIME_4CYCLES,  /*!< 采样周期为4个周期 */
    ADC_SAMPLETIME_5CYCLES,  /*!< 采样周期为5个周期 */
    ADC_SAMPLETIME_6CYCLES,  /*!< 采样周期为6个周期 */
    ADC_SAMPLETIME_7CYCLES,  /*!< 采样周期为7个周期 */
    ADC_SAMPLETIME_8CYCLES,  /*!< 采样周期为8个周期 */
    ADC_SAMPLETIME_9CYCLES,  /*!< 采样周期为9个周期 */
    ADC_SAMPLETIME_10CYCLES, /*!< 采样周期为10个周期 */
    ADC_SAMPLETIME_11CYCLES, /*!< 采样周期为11个周期 */
    ADC_SAMPLETIME_12CYCLES, /*!< 采样周期为12个周期 */
    ADC_SAMPLETIME_13CYCLES, /*!< 采样周期为13个周期 */
    ADC_SAMPLETIME_14CYCLES, /*!< 采样周期为14个周期 */
    ADC_SAMPLETIME_15CYCLES, /*!< 采样周期为15个周期 */
    ADC_SAMPLETIME_16CYCLES  /*!< 采样周期为16个周期 */
} adc_sampletime_t;

/*!
 * @brief   ADC输入通道选择
 */
typedef enum
{
    ADC_CHANNEL_0    = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(0),  /*!< 外部输入通道0 */
    ADC_CHANNEL_1    = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(1),  /*!< 外部输入通道1 */
    ADC_CHANNEL_2    = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(2),  /*!< 外部输入通道2 */
    ADC_CHANNEL_3    = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(3),  /*!< 外部输入通道3 */
    ADC_CHANNEL_4    = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(4),  /*!< 外部输入通道4 */
    ADC_CHANNEL_5    = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(5),  /*!< 外部输入通道5 */
    ADC_CHANNEL_6    = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(6),  /*!< 外部输入通道6 */
    ADC_CHANNEL_7    = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(7),  /*!< 外部输入通道7 */
    ADC_CHANNEL_8    = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(8),  /*!< 外部输入通道8 */
    ADC_CHANNEL_9    = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(9),  /*!< 外部输入通道9 */
    ADC_CHANNEL_10   = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(10), /*!< 外部输入通道10 */
    ADC_CHANNEL_11   = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(11), /*!< 外部输入通道11 */
    ADC_CHANNEL_12   = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(12), /*!< 外部输入通道12 */
    ADC_CHANNEL_13   = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(13), /*!< 外部输入通道13 */
    ADC_CHANNEL_14   = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(14), /*!< 外部输入通道14 */
    ADC_CHANNEL_15   = ADC_CTL_ECH_TYPE_SEL(0) | ADC_CTL_ECH_SEL(15), /*!< 外部输入通道15 */
    ADC_CHANNEL_16   = ADC_CTL_ECH_TYPE_SEL(1) | ADC_CTL_ECH_SEL(0),  /*!< 外部输入通道16 */
    ADC_CHANNEL_17   = ADC_CTL_ECH_TYPE_SEL(1) | ADC_CTL_ECH_SEL(1),  /*!< 外部输入通道17 */
    ADC_CHANNEL_18   = ADC_CTL_ECH_TYPE_SEL(1) | ADC_CTL_ECH_SEL(2),  /*!< 外部输入通道18 */
    ADC_CHANNEL_19   = ADC_CTL_ECH_TYPE_SEL(1) | ADC_CTL_ECH_SEL(3),  /*!< 外部输入通道19 */
    ADC_CHANNEL_20   = ADC_CTL_ECH_TYPE_SEL(1) | ADC_CTL_ECH_SEL(4),  /*!< 外部输入通道20 */
    ADC_CHANNEL_21   = ADC_CTL_ECH_TYPE_SEL(1) | ADC_CTL_ECH_SEL(5),  /*!< 外部输入通道21 */
    ADC_CHANNEL_22   = ADC_CTL_ECH_TYPE_SEL(1) | ADC_CTL_ECH_SEL(6),  /*!< 外部输入通道22 */
    ADC_CHANNEL_23   = ADC_CTL_ECH_TYPE_SEL(1) | ADC_CTL_ECH_SEL(7),  /*!< 外部输入通道23 */
    ADC_CHANNEL_24   = ADC_CTL_ECH_TYPE_SEL(1) | ADC_CTL_ECH_SEL(8),  /*!< 外部输入通道24 */
    ADC_CHANNEL_25   = ADC_CTL_ECH_TYPE_SEL(1) | ADC_CTL_ECH_SEL(9),  /*!< 外部输入通道25 */
    ADC_CHANNEL_26   = ADC_CTL_ECH_TYPE_SEL(1) | ADC_CTL_ECH_SEL(10), /*!< 外部输入通道26 */
    ADC_CHANNEL_27   = ADC_CTL_ECH_TYPE_SEL(1) | ADC_CTL_ECH_SEL(11), /*!< 外部输入通道27 */
    ADC_CHANNEL_AVDD = ADC_CTL_ICH_SEL(1),                            /*!< 内部输入通道，连接到AVDD/2 */
    ADC_CHANNEL_VBUF = ADC_CTL_ICH_SEL(2)                             /*!< 内部输入通道，连接到VBUF */
} adc_channel_t;

/*!
 * @brief  时钟分频
 */
typedef enum
{
    ADC_CLOCK_DIV2,  /*!< 2分频 */
    ADC_CLOCK_DIV4,  /*!< 4分频 */
    ADC_CLOCK_DIV6,  /*!< 6分频 */
    ADC_CLOCK_DIV8,  /*!< 8分频 */
    ADC_CLOCK_DIV12, /*!< 12分频 */
    ADC_CLOCK_DIV16, /*!< 16分频 */
    ADC_CLOCK_DIV20, /*!< 20分频 */
    ADC_CLOCK_DIV32  /*!< 32分频 */
} adc_clock_prescaler_t;

/*!
 * @brief   连续转换
 */
typedef enum
{
    ADC_CONV_SINGLE,    /*!< 单次转换 */
    ADC_CONV_CONTINUOUS /*!< 连续转换 */
} adc_continuous_mode_t;

/*!
 * @brief   触发源选择
 */
typedef enum
{
    ADC_TRIG_SOFT,    /*!< 内部(软件)触发 */
    ADC_TRIG_EXTERNAL /*!< 外部(硬件)触发功能 */
} adc_trig_mode_t;

/*!
 * @brief   ADC外部触发边沿选择
 */
typedef enum
{
    ADC_TRIG_EDGE_RISING, /*!< 上升沿触发 */
    ADC_TRIG_EDGE_FALLING /*!< 下降沿触发 */
} adc_trig_edge_t;

/*!
 * @brief   触发延时，从触发到ADC启动采用转换的延时，以MCLK cycle为单位
 */
typedef enum
{
    ADC_TRGDL_1CYCLE,   /*!< 触发延时 1 MCLK */
    ADC_TRGDL_2CYCLES,  /*!< 触发延时 2 MCLKS */
    ADC_TRGDL_4CYCLES,  /*!< 触发延时 4 MCLKS */
    ADC_TRGDL_8CYCLES,  /*!< 触发延时 8 MCLKS */
    ADC_TRGDL_12CYCLES, /*!< 触发延时 12 MCLKS */
    ADC_TRGDL_16CYCLES, /*!< 触发延时 16 MCLKS */
    ADC_TRGDL_24CYCLES, /*!< 触发延时 24 MCLKS */
    ADC_TRGDL_32CYCLES  /*!< 触发延时 32 MCLKS */
} adc_trigger_delay_t;

/*!
 * @brief   中断标志
 */
typedef enum
{
    ADC_INT_COMPARE = ADC_IER_REFIE_Msk, /* 数据比较中断 */
    ADC_INT_FINISH  = ADC_IER_EOCIE_Msk  /* 转换结束中断 */
} adc_interrupt_t;

/*!
 * @brief   状态标志
 */
typedef enum
{
    ADC_FLAG_COMPARE = ADC_ISR_REFIF_Msk, /* 转换结果比较状态 */
    ADC_FLAG_FINISH  = ADC_ISR_EOCIF_Msk  /*转化完成*/
} adc_status_flag_t;

/*!
 * @brief   通道配置
 */
typedef struct
{
    adc_sampletime_t sampling_time;         /*!< 采样时间 */
    adc_trigger_delay_t trigger_delay_time; /*!< 触发延时时间 */
} adc_channel_config_t;

/*!
 * @brief 硬件比较配置
 */
typedef struct
{
    adc_compare_mode_t mode; /*!< 比较模式 */
    uint16_t high_limit;     /*!< 比较上限值 */
    uint16_t low_limit;      /*!< 比较下限值 */
} adc_compare_config_t;

/*!
 * @brief   ADC配置结构体
 */
typedef struct
{
    adc_voltage_reference_t vref;          /*!< 参考电压 */
    adc_clock_prescaler_t prescaler;       /*!< 时钟预分频 */
    adc_trig_mode_t trigger;               /*!< 触发选择 */
    adc_trig_edge_t edge;                  /*!< 触发边沿选择 */
    adc_continuous_mode_t continuous_conv; /*!< 连续转换 */
} adc_config_t;

/*******************************************************************************
 * API
 ******************************************************************************/
/*!
 * @name ADC驱动API
 * @{
 */

#if defined(__cplusplus)
extern "C" {
#endif
void ADC_DeInit(void);
void ADC_Init(const adc_config_t *config);
void ADC_StartChannel(adc_channel_t channel, const adc_channel_config_t *config);
void ADC_Stop(void);
void ADC_SetVref(adc_voltage_reference_t vref);
uint32_t ADC_GetValue(void);
void ADC_TrigSwConversion(void);
void ADC_SetCompare(const adc_compare_config_t *config);
void ADC_SetIntMode(adc_interrupt_t interrupt, bool enable);
bool ADC_GetStatusFlag(adc_status_flag_t flag);
void ADC_ClearStatusFlag(adc_status_flag_t flag);
bool ADC_GetIntFlag(adc_status_flag_t interrupt);
#if defined(__cplusplus)
}
#endif

/*! @} */

#endif /* NCC0670_ADC_H */
/*******************************************************************************
 * EOF
 ******************************************************************************/
