/*! @file    nccm0670_flash.h
 *  @brief   FLASH固件库
 *  @version 2021-04-25  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_FLASH_H
#define NCC0670_FLASH_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_flash
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/
typedef enum
{
    FLASH_FLAG_SUCCESS,               /*!< 操作正常完成，无错误 */
    FLASH_FLAG_ERR_PROTECT_AREA,      /*!< 编程受保护的区域 */
    FLASH_FLAG_ERR_PROTECT_PAGE,      /*!< 擦除受保护的页 */
    FLASH_FLAG_ERR_PROTECT_SECTOR,    /*!< 擦除受保护的扇区 */
    FLASH_FLAG_ERR_USER_AREA_DISABLE, /*!< 用户配置区擦除未使能时，擦除用户配置区 */
    FLASH_FLAG_ERR_USER_AREA_ENABLE,  /*!< 用户配置区擦拭使能，但有其他区域受保护时，擦除用户配置区 */
    FLASH_FLAG_ERR_NVR1,              /*!< 禁止按扇区擦除NVR1扇区 */
    FLASH_FLAG_ERR_CHIP_ERASE_DENIED, /*!< 用户模式下禁止全擦操作 */
    FLASH_FLAG_ERR_WRITE_TIMEOUT,     /*!< FLASH写操作超时 */
    FLASH_FLAG_ERR_OUTOF_MEMORY,      /*!< 编程或擦除超过flash的边界 */
} flash_status_flag_t;

/*******************************************************************************
 * API
 ******************************************************************************/
#if defined(__cplusplus)
extern "C" {
#endif

bool FLASH_BlockErase(uint32_t page_adrress);
bool FLASH_PageErase(uint32_t page_adrress);
bool FLASH_ProgramWord(uint32_t address, uint32_t data);
bool FLASH_GetStatusFlag(flash_status_flag_t flag);
void FLASH_ClearStatusFlag(flash_status_flag_t flag);

#if defined(__cplusplus)
}
#endif

/*! @} */

#endif /* NCC0670_FLASH_H */
/*******************************************************************************
 * EOF
 ******************************************************************************/
