/*! @file    nccm0670_iwdg.h
 *  @brief   IWDG固件库
 *  @version 2022-11-03  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_IWDG_H
#define NCC0670_IWDG_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_iwdg
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/*!
 * @brief   预分频系数
 */
typedef enum
{
    IWDG_PRESCALER_1,  /*!< 不分频   */
    IWDG_PRESCALER_4,  /*!< 4分频   */
    IWDG_PRESCALER_16, /*!< 16分频   */
    IWDG_PRESCALER_64  /*!< 64分频   */
} iwdg_prescaler_t;

/*!
 * @brief   IWDG配置结构体
 */
typedef struct
{
    iwdg_prescaler_t prescaler; /*!< 预分频系数*/
    uint16_t reload_value;      /*!< 重载计数数值 */
} iwdg_config_t;

/*******************************************************************************
 * API
 ******************************************************************************/
#if defined(__cplusplus)
extern "C" {
#endif

void IWDG_Init(const iwdg_config_t *const config);
void IWDG_Refresh(void);
void IWDG_EnableWriteAccess(void);
void IWDG_Enable(void);

#if defined(__cplusplus)
}
#endif

/*! @} */

#endif /* NCC0670_IWDG_H */
/*******************************************************************************
 * EOF
 ******************************************************************************/
