/*! @file    nccm0670_misc.h
 *  @brief   MISC固件库
 *  @version 2022-11-03  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_MISC_H
#define NCC0670_MISC_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_misc
 * @{
 */
/*******************************************************************************
 * Definitions
 ******************************************************************************/

#define SCB_LPM_SLEEP_EXIT_ISR  SCB_SCR_SLEEPONEXIT_Msk /*!< 退出中断后进入低功耗 */
#define SCB_LPM_DEEPSLEEP       SCB_SCR_SLEEPDEEP_Msk   /*!< 进入DEEPSLEEP模式 */
#define SCB_LPM_WAKE_BY_ALL_INT SCB_SCR_SEVONPEND_Msk   /*!< 任意中断唤醒 */

/*!
 * @brief systick时钟源
 */
typedef enum
{
    SYSTICK_CLKSOURCE_HCLK_DIV8, /*!< HCLK 时钟除以 8 当作systick时钟源 */
    SYSTICK_CLKSOURCE_HCLK       /*!< HCLK 时钟当作systick时钟源 */
} systick_clk_source_t;

/*******************************************************************************
 * API
 ******************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif

void LPM_EnableSleep(uint8_t lowpower_mode);
void LPM_DisableSleep(uint8_t lowpower_mode);
void SYSTICK_SetClkSource(systick_clk_source_t source);

#if defined(__cplusplus)
}
#endif
/*! @} */
#endif /* NCC0670_MISC_H*/
/*******************************************************************************
 * EOF
 ******************************************************************************/
