/*! @file    nccm0670_tk.h
 *  @brief   TK固件库
 *  @version 2021-04-13  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_TK_H
#define NCC0670_TK_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_tk
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/*!
 * @brief TK通道选择
 */
#define TK_CHANNEL_0   (0x01U << 0)  /*!< 触摸通道0  */
#define TK_CHANNEL_1   (0x01U << 1)  /*!< 触摸通道1  */
#define TK_CHANNEL_2   (0x01U << 2)  /*!< 触摸通道2  */
#define TK_CHANNEL_3   (0x01U << 3)  /*!< 触摸通道3  */
#define TK_CHANNEL_4   (0x01U << 4)  /*!< 触摸通道4  */
#define TK_CHANNEL_5   (0x01U << 5)  /*!< 触摸通道5  */
#define TK_CHANNEL_6   (0x01U << 6)  /*!< 触摸通道6  */
#define TK_CHANNEL_7   (0x01U << 7)  /*!< 触摸通道7  */
#define TK_CHANNEL_8   (0x01U << 8)  /*!< 触摸通道8  */
#define TK_CHANNEL_9   (0x01U << 9)  /*!< 触摸通道9  */
#define TK_CHANNEL_10  (0x01U << 10) /*!< 触摸通道10 */
#define TK_CHANNEL_11  (0x01U << 11) /*!< 触摸通道11 */
#define TK_CHANNEL_12  (0x01U << 12) /*!< 触摸通道12 */
#define TK_CHANNEL_13  (0x01U << 13) /*!< 触摸通道13 */
#define TK_CHANNEL_14  (0x01U << 14) /*!< 触摸通道14 */
#define TK_CHANNEL_15  (0x01U << 15) /*!< 触摸通道15 */
#define TK_CHANNEL_ALL 0xFFFFU       /*!< 全部触摸通道 */

/*!
 * @brief TK触发ADC模式
 */
typedef enum
{
    TK_TRIG_HARDWARE, /*!< 硬件自动触发ADC转换 */
    TK_TRIG_SOFTWARE  /*!< 软件配置触发ADC转换 */
} tk_trig_mode_t;

/*!
 * @brief TK工作时钟选择
 */
typedef enum
{
    TK_CLK_SRC_HSI,     /*!< 选择内部高速时钟 48MHz */
    TK_CLK_SRC_LSI512K, /*!< 选择较快内部低振 512KHz */
    TK_CLK_SRC_LSI,     /*!< 选择内部低振 32KHz */
    TK_CLK_SRC_LSE      /*!< 选择外部低振 32KHz */
} tk_clk_source_t;

/*!
 * @brief 按键充放电分频
 */
typedef enum
{
    TK_CHARGE_FREQ_DIV_6,   /*!< 工作时钟6分频 */
    TK_CHARGE_FREQ_DIV_8,   /*!< 工作时钟8分频 */
    TK_CHARGE_FREQ_DIV_12,  /*!< 工作时钟12分频 */
    TK_CHARGE_FREQ_DIV_24,  /*!< 工作时钟24分频 */
    TK_CHARGE_FREQ_DIV_48,  /*!< 工作时钟48分频 */
    TK_CHARGE_FREQ_DIV_96,  /*!< 工作时钟96分频 */
    TK_CHARGE_FREQ_DIV_192, /*!< 工作时钟192分频 */
    TK_CHARGE_FREQ_DIV_384  /*!< 工作时钟384分频 */
} tk_charge_freq_div_t;

/*!
 * @brief 按键两通道一次完整充放电之间的间隔时间
 */
typedef enum
{
    TK_CHARGE_INTERVAL_62P5US,  /*!< 间隔时间62.5us */
    TK_CHARGE_INTERVAL_125US,   /*!< 间隔时间125us */
    TK_CHARGE_INTERVAL_187P5US, /*!< 间隔时间187.5us */
    TK_CHARGE_INTERVAL_250US,   /*!< 间隔时间250us */
    TK_CHARGE_INTERVAL_312P5US, /*!< 间隔时间312.5us */
    TK_CHARGE_INTERVAL_375US,   /*!< 间隔时间375us */
    TK_CHARGE_INTERVAL_437P5US, /*!< 间隔时间437.5us */
    TK_CHARGE_INTERVAL_500US,   /*!< 间隔时间500us */
    TK_CHARGE_INTERVAL_750US,   /*!< 间隔时间750us */
    TK_CHARGE_INTERVAL_1MS,     /*!< 间隔时间1ms */
    TK_CHARGE_INTERVAL_1P5MS,   /*!< 间隔时间1.5ms */
    TK_CHARGE_INTERVAL_2MS,     /*!< 间隔时间2ms */
    TK_CHARGE_INTERVAL_2P5MS,   /*!< 间隔时间2.5ms */
    TK_CHARGE_INTERVAL_3MS,     /*!< 间隔时间3ms */
    TK_CHARGE_INTERVAL_3P5MS,   /*!< 间隔时间3.5ms */
    TK_CHARGE_INTERVAL_4MS      /*!< 间隔时间4ms */
} tk_charge_interval_t;

/*!
 * @brief 低功耗时按键的检测匹配次数
 */
typedef enum
{
    TK_CRITERION_PASS_ALL,    /*!< 按键检测后的ADC转换数据都符合设定的阈值 */
    TK_CRITERION_FAIL_1_MOST, /*!< 按键检测后的ADC转换数据最多1次未符合设定的阈值 */
    TK_CRITERION_FAIL_2_MOST, /*!< 按键检测后的ADC转换数据最多2次未符合设定的阈值 */
    TK_CRITERION_FAIL_3_MOST  /*!< 按键检测后的ADC转换数据最多3次未符合设定的阈值 */
} tk_criterion_mode_t;

/*!
 * @brief   中断标志
 */
typedef enum
{
    TK_INT_CHARGE_FINISHED = TK_CR_TK_CRGIE_Msk,     /* 充放电结束中断 */
    TK_INT_WKUP = TK_CR_TK_MATIE_Msk,                /* 按键唤醒中断 */
    TK_INT_CONVERT_UNFINISHED = TK_CR_TK_NECOVIE_Msk /* 充放电时间中ADC转换未完成中断 */
} tk_interrupt_t;

/*!
 * @brief TK中断标志位
 */
typedef enum
{
    TK_FLAG_CHARGE_FINISHED = TK_ISR_TK_CRGF_Msk,     /*!< 充放电完成中断  */
    TK_FLAG_WKUP_BY_CH0 = TK_ISR_TK_CH0_MATF_Msk,     /*!< 低功耗模式下通道0匹配通过  */
    TK_FLAG_WKUP_BY_CH1 = TK_ISR_TK_CH1_MATF_Msk,     /*!< 低功耗模式下通道1匹配通过  */
    TK_FLAG_WKUP_BY_CH2 = TK_ISR_TK_CH2_MATF_Msk,     /*!< 低功耗模式下通道2匹配通过  */
    TK_FLAG_WKUP_BY_CH3 = TK_ISR_TK_CH3_MATF_Msk,     /*!< 低功耗模式下通道3匹配通过  */
    TK_FLAG_WKUP_BY_CH4 = TK_ISR_TK_CH4_MATF_Msk,     /*!< 低功耗模式下通道4匹配通过  */
    TK_FLAG_WKUP_BY_CH5 = TK_ISR_TK_CH5_MATF_Msk,     /*!< 低功耗模式下通道5匹配通过  */
    TK_FLAG_WKUP_BY_CH6 = TK_ISR_TK_CH6_MATF_Msk,     /*!< 低功耗模式下通道6匹配通过  */
    TK_FLAG_WKUP_BY_CH7 = TK_ISR_TK_CH7_MATF_Msk,     /*!< 低功耗模式下通道7匹配通过  */
    TK_FLAG_WKUP_BY_CH8 = TK_ISR_TK_CH8_MATF_Msk,     /*!< 低功耗模式下通道8匹配通过  */
    TK_FLAG_WKUP_BY_CH9 = TK_ISR_TK_CH9_MATF_Msk,     /*!< 低功耗模式下通道9匹配通过  */
    TK_FLAG_WKUP_BY_CH10 = TK_ISR_TK_CH10_MATF_Msk,   /*!< 低功耗模式下通道10匹配通过  */
    TK_FLAG_WKUP_BY_CH11 = TK_ISR_TK_CH11_MATF_Msk,   /*!< 低功耗模式下通道11匹配通过  */
    TK_FLAG_WKUP_BY_CH12 = TK_ISR_TK_CH12_MATF_Msk,   /*!< 低功耗模式下通道12匹配通过  */
    TK_FLAG_WKUP_BY_CH13 = TK_ISR_TK_CH13_MATF_Msk,   /*!< 低功耗模式下通道13匹配通过  */
    TK_FLAG_WKUP_BY_CH14 = TK_ISR_TK_CH14_MATF_Msk,   /*!< 低功耗模式下通道14匹配通过  */
    TK_FLAG_WKUP_BY_CH15 = TK_ISR_TK_CH15_MATF_Msk,   /*!< 低功耗模式下通道15匹配通过  */
    TK_FLAG_CONVERT_UNFINISHED = TK_ISR_TK_NECOVF_Msk /*!< 充放电时间中ADC转换未完成 */
} tk_status_flag_t;

/*******************************************************************************
 * API
 ******************************************************************************/

#if defined(__cplusplus)
extern "C" {
#endif

void TK_DeInit(void);
void TK_SetChannels(uint16_t channel);
void TK_SetTrigMode(tk_trig_mode_t mode);
uint16_t TK_GetWorkingChannel(void);
void TK_SetClockSource(tk_clk_source_t clk);
void TK_SetChargeCycles(uint8_t times);
void TK_SetChargeInterval(tk_charge_interval_t interval);
void TK_SetChargeFreq(tk_charge_freq_div_t freq);
void TK_SetDischarge(bool enable);
void TK_SetHwTrigLeadTime(uint16_t time);
void TK_Enable();
void TK_Disable();

void TK_SetWakeupThreshold(uint16_t channel, uint32_t threshold);
void TK_SetWakeupCriterion(uint8_t num, tk_criterion_mode_t mode);

bool TK_GetStatusFlag(tk_status_flag_t flag);
void TK_ClearStatusFlag(tk_status_flag_t flag);
void TK_SetIntMode(tk_interrupt_t interrupt, bool enable);
bool TK_GetIntFlag(tk_status_flag_t flag);

#if defined(__cplusplus)
}
#endif
/*! @} */
#endif /* NCC0670_TK_H*/
/*******************************************************************************
 * EOF
 ******************************************************************************/
