/*! @file    nccm0670_wwdg.h
 *  @brief   WWDG固件库
 *  @version 2021-04-14  V1.0.0
 *
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef NCC0670_WWDG_H
#define NCC0670_WWDG_H
#include "nccm0670.h"
/*!
 * @addtogroup nccm0670_wwdg
 * @{
 */

/*******************************************************************************
 * Definitions
 ******************************************************************************/
/*!
 * @brief   预分频器
 */
typedef enum
{
    WWDG_PRESCALER_1, /*!< WWDG 计数时钟 = (PCLK1/4096)/1 */
    WWDG_PRESCALER_2, /*!< WWDG 计数时钟 = (PCLK1/4096)/2 */
    WWDG_PRESCALER_4, /*!< WWDG 计数时钟 = (PCLK1/4096)/4 */
    WWDG_PRESCALER_8  /*!< WWDG 计数时钟 = (PCLK1/4096)/8 */
} wwdg_prescaler_t;

/*!
 * @brief   WWDG配置结构体
 */
typedef struct
{
    wwdg_prescaler_t prescaler; /*!< 预分频系数 */
    uint32_t window;            /*!< 窗口值，范围：0x40 <= 窗口值 <= 0x7F */
    uint32_t counter;           /*!< 计数器，范围：0x40 <= 窗口值 <= 0x7F */
    uint32_t ewi_mode;          /*!< 提前唤醒中断使能 */
} wwdg_config_t;

/*!
 * @brief   WWDG中断
 */
typedef enum
{
    WWDG_INT_EWI = WWDG_CFR_EWI_Msk /*!< 提前唤醒中断 */
} wwdg_interrupt_t;

/*!
 * @brief   WWDG状态
 */
typedef enum
{
    WWDG_FLAG_EWI = WWDG_SR_EWIF_Msk /*!< 提前唤醒标志 */
} wwdg_status_flag_t;

/*******************************************************************************
 * API
 ******************************************************************************/
#if defined(__cplusplus)
extern "C" {
#endif

void WWDG_DeInit(void);
void WWDG_Init(const wwdg_config_t *config);
void WWDG_SetCounter(uint32_t counter);
void WWDG_Enable(void);
void WWDG_SetIntMode(wwdg_interrupt_t interrupt, bool enable);
bool WWDG_GetStatusFlag(wwdg_status_flag_t flag);
void WWDG_ClearStatusFlag(wwdg_status_flag_t flag);

#if defined(__cplusplus)
}
#endif

/*! @} */

#endif /* NCC0670_WWDG_H */
/*******************************************************************************
 * EOF
 ******************************************************************************/
