/*
 * Copyright (C) 2020-2030 NCC Limited. All rights reserved.
 *
 * THIS SOFTWARE IS PROVIDED BY NCC "AS IS" AND ANY EXPRESSED OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL NCC OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/*!
 * @defgroup nccm0670_misc MISC
 * @brief MISC模块固件库

## 详细说明 ##

1. 进入和退出低功耗模式
2. systick时钟源设置

 * @{
 */
#include "nccm0670_misc.h"

/*!
 * @brief  使能低功耗模式配置
 *
 * @param[in] lowpower_mode 低功耗模式
 *  @arg SCB_LPM_SLEEP_EXIT_ISR  退出中断后进入低功耗
 *  @arg SCB_LPM_DEEPSLEEP       进入DEEPSLEEP模式
 *  @arg SCB_LPM_WAKE_BY_ALL_INT 任意中断唤醒
 * @return None
 */
void LPM_EnableSleep(uint8_t lowpower_mode)
{
    SCB->SCR |= (uint32_t)lowpower_mode;
}

/*!
 * @brief  关闭低功耗模式配置
 *
 * @param[in] lowpower_mode 低功耗模式
 *  @arg SCB_LPM_SLEEP_EXIT_ISR  退出中断后进入低功耗
 *  @arg SCB_LPM_DEEPSLEEP       进入DEEPSLEEP模式
 *  @arg SCB_LPM_WAKE_BY_ALL_INT 任意中断唤醒
 * @return None
 */
void LPM_DisableSleep(uint8_t lowpower_mode)
{
    SCB->SCR &= (~(uint32_t)lowpower_mode);
}

/*!
 * @brief  设置系统定时器时钟来源
 * @param[in] source 时钟源选择
 * @return None
 */
void SYSTICK_SetClkSource(systick_clk_source_t source)
{
    if (source == SYSTICK_CLKSOURCE_HCLK)
    {
        SysTick->CTRL |= SYSTICK_CLKSOURCE_HCLK;
    }
    else
    {
        SysTick->CTRL &= ~SYSTICK_CLKSOURCE_HCLK;
    }
}
/*! @} */
